var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { uniq } from "lodash";
import * as debug from "debug";
const log = debug("kds:pie-api-player:base-element");
export const DEFAULT_HOST = "https://dev-service.pie-api.com/endpoints";
export const KDS_CSS = "https://s3.amazonaws.com/pie-api-player-dev/content-styles.css";
export const loadCss = () => {
    const kdsCss = document.head.querySelector(`link[href="${KDS_CSS}"]`);
    if (!kdsCss) {
        const link = document.createElement("link");
        link.setAttribute("href", KDS_CSS);
        link.setAttribute("rel", "stylesheet");
        document.head.appendChild(link);
    }
    else {
        log("kds css already loaded");
    }
};
export class BaseElement extends HTMLElement {
    get itemId() {
        return this.getAttribute("item-id");
    }
    get token() {
        return this.getAttribute("token");
    }
    // TODO: rename host to something a bit more accurate - it's not just a host it's a full url.
    get host() {
        return this.getAttribute("host") || DEFAULT_HOST;
    }
    connectedCallback() {
        loadCss();
    }
    /**
     * just load the scripts asynchronously..
     * we're going to use customElements.whenDefined to know when things are ready.
     */
    loadJs(view, elements) {
        return __awaiter(this, void 0, void 0, function* () {
            view.forEach(src => {
                const script = document.createElement("script");
                script.src = src;
                document.head.appendChild(script);
            });
            const elementNames = uniq(elements);
            log("elementNames: ", elementNames);
            return Promise.all(elementNames.map(e => customElements.whenDefined(e)));
        });
    }
}
//# sourceMappingURL=base-element.js.map