import { PlayerApiClient } from "../player";
import { assertFetch } from "./utils";
jest.mock("node-fetch", () => ({
    default: jest.fn().mockReturnValue(Promise.resolve({
        status: 200,
        json: jest.fn().mockReturnValue(Promise.resolve({}))
    }))
}));
const endpoints = {
    saveSession: { method: "POST", path: "/save-session" },
    model: { method: "POST", path: "/model" },
    score: { method: "POST", path: "/score" }
};
const session = {
    id: "1",
    data: []
};
const client = () => new PlayerApiClient("host", "token", endpoints);
describe("player", () => {
    describe("load", () => {
        assertFetch(client(), c => c.load("itemId", "sessionId"), `host/player/load.json`, {
            itemId: "itemId",
            sessionId: "sessionId"
        });
    });
    describe("model", () => {
        assertFetch(client(), c => c.model("itemId", session, {}), `host${endpoints.model.path}`, 
        // tslint:disable-next-line:object-literal-shorthand
        { itemId: "itemId", session: session, env: {} });
    });
    describe("score", () => {
        assertFetch(client(), c => c.score("itemId", session), `host${endpoints.score.path}`, 
        // tslint:disable-next-line:object-literal-shorthand
        { itemId: "itemId", session: session, disablePartialScoring: false });
    });
    describe("saveSession", () => {
        assertFetch(client(), c => c.saveSession({ id: "id", data: [] }), `host${endpoints.saveSession.path}`, { id: "id", data: [] });
    });
});
//# sourceMappingURL=client.test.js.map