var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as debug from "debug";
import { SessionChangedEvent } from "@pie-framework/pie-player-events";
import { loadCss } from "./base-element";
const log = debug("pie-api:config-and-preview");
class ToggleButton extends HTMLElement {
    static get observedAttributes() {
        return ["toggled"];
    }
    get toggled() {
        return this.getAttribute("toggled") === "true";
    }
    get hideMsg() {
        return this.getAttribute("hide-message") || "hide";
    }
    get showMsg() {
        return this.getAttribute("show-message") || "show";
    }
    attributeChangedCallback(name, newValue, oldValue) {
        if (name === "toggled" && newValue !== oldValue) {
            this.update();
        }
    }
    connectedCallback() {
        this.innerHTML = `<style>
      .toggle-button {
        cursor: pointer;
        border: none;
        outline: none;
        background-color: lightgrey;
      } 
    </style>
    <button class="toggle-button"></button>`;
        this.$button = this.querySelector("button");
        this.$button.addEventListener("click", () => {
            this.dispatchEvent(new CustomEvent("toggle", {
                bubbles: true,
                detail: {
                    toggled: this.toggled
                }
            }));
        });
        this.update();
    }
    update() {
        if (this.$button) {
            log("[update] this.toggled: ", this.toggled);
            this.$button.textContent = this.toggled ? this.hideMsg : this.showMsg;
        }
    }
}
customElements.define("toggle-button", ToggleButton);
export default class ConfigAndPreview extends HTMLElement {
    constructor() {
        super();
        this.showPreview = false;
        this._session = {};
        this.showPreview = true;
    }
    set mode(m) {
        this._mode = m;
        this.updateRender(this._model, false);
    }
    updateRender(model, reset) {
        return __awaiter(this, void 0, void 0, function* () {
            log("[updateRender] reset: ", reset);
            if (reset) {
                this._session = {};
            }
            const renderModel = yield this._controller.model(model, this._session, {
                mode: this._mode
            });
            log("[updateRender] renderModel: ", renderModel);
            if (this.$render) {
                this.$render.model = renderModel;
            }
        });
    }
    connectedCallback() {
        loadCss();
        // this.render();
    }
    render() {
        if (!this._model || !this._controller || !this._element) {
            log("[render] not ready yet - return, model: ", this._model, "controller: ", this._controller, "element: ", this._element);
            return;
        }
        log("[render] everything ready - begin loading...");
        const configureName = `${this._element}-configure`;
        log("[render] els?", customElements.get(this._element), customElements.get(configureName));
        Promise.all([configureName, this._element].map(n => customElements.whenDefined(n)))
            .then(() => __awaiter(this, void 0, void 0, function* () {
            log("[render] custom elements are ready!");
            this.innerHTML = `
      <style>

          .control-bar{
            width: 100%;
            display: flex;
            justify-content: space-between;
            font-size: 16px;
            font-weight: bold;
            color: grey;
            border-bottom: solid 1px lightgrey;
            padding-top: 10px;
            padding-bottom: 10px;
          }

          #configure {
            flex: 1 0 0;
          }

          #render {
            flex: 1 0 0;
          }

          .root {
            border: green;
          }

          .config-holder{
            display: flex;
            width: 100%;
            padding-top: 20px;
            padding-bottom: 20px;
          }
          .divider {
            width: 1px;
            border-right: solid 1px lightgrey;
            margin-left: 20px;
            margin-right: 20px;
          }
      </style>
      <div class="root">
        <span class="control-bar">
          <span class="bar"> </span>
          <span>
          <toggle-button 
          id="preview-toggle"
          show-message="Show preview"
          hide-message="Hide preview"
          toggled="${this.showPreview}"
          ></toggle-button>
          <button id="delete-button">delete</button>
          </span>
        </span>
        <div class="config-holder">
          <${configureName} id="configure"></${configureName}>
          <div class="divider"></div>
          <${this._element} id="render"></${this._element}>
        </div>
      </div>`;
            /**
             * TODOs:
             * - debug info should be showable/hideable
             * - preview should be showable/hidable
             * - should be able to see the raw session/model too
             */
            this.querySelector(".bar").innerHTML = `${this._element} #${this._model.id}`;
            log("[render] find elements...");
            this.$divider = this.querySelector(".divider");
            this.$toggle = this.querySelector("#preview-toggle");
            this.$deleteButton = this.querySelector("#delete-button");
            this.$toggle.addEventListener("toggle", (event) => {
                event.target.setAttribute("toggled", !event.detail.toggled);
                this.showPreview = !event.detail.toggled;
                this.$toggle.setAttribute("toggled", (!event.detail.toggled).toString());
                this.updatePreview();
            });
            this.$deleteButton.addEventListener("click", () => {
                this.dispatchEvent(new CustomEvent("delete-element", {
                    bubbles: true,
                    detail: {
                        id: this.getAttribute("pie-id")
                    }
                }));
            });
            this.$configure = this.querySelector("#configure");
            this.$render = this.querySelector("#render");
            this.$configure.model = this._model;
            this.$render.session = this._session;
            log("[render] addEventListeners...");
            this.$render.addEventListener(SessionChangedEvent.TYPE, (e) => {
                log("session changed!", this._session);
            });
            this.$configure.addEventListener("model.updated", (e) => {
                log("model updated: ", e);
                this._model = e.detail.update;
                this.updateRender(e.detail.update, e.detail.reset);
            });
            log("[render] call updateRender");
            this.updateRender(this._model, false);
        }))
            .catch(e => {
            console.error(e);
        });
    }
    updatePreview() {
        if (this.showPreview) {
            this.$render.setAttribute("style", "display: block");
            this.$divider.setAttribute("style", "display: block");
        }
        else {
            this.$render.setAttribute("style", "display: none");
            this.$divider.setAttribute("style", "display: none");
        }
    }
    set model(m) {
        this._model = m;
        this.render();
    }
    set controller(c) {
        this._controller = c;
        this.render();
    }
    set elementName(n) {
        this._element = n;
        this.render();
    }
}
//# sourceMappingURL=config-and-preview.js.map