var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { BaseElement } from "./base-element";
import { ConfigureClient } from "./clients/configure";
import * as debug from "debug";
import { ModelUpdatedEvent } from "@pie-framework/pie-configure-events";
import { callPromise } from "./helpers/promise";
import createDataModel from "./default-data";
const CONTROLLER_GLOBAL = "pie-controller-pie-item";
const log = debug("pie-api:player:configure");
const toTag = m => {
    const tag = `${m.element}-configure`;
    return `<${tag} pie-id="${m.id}"></${tag}>`;
};
class ErrorMessage extends HTMLElement {
    set message(m) {
        this._message = m;
        this.showError();
    }
    get timeout() {
        const t = this.getAttribute("timeout");
        const parsed = parseInt(t, 10);
        return Number.isFinite(parsed) ? parsed : 10000;
    }
    showError() {
        this.$root.innerHTML = this._message;
        this.$root.setAttribute("class", "error-root show");
        setTimeout(() => {
            this.$root.setAttribute("class", "error-root hide");
        }, this.timeout);
    }
    connectedCallback() {
        this.innerHTML = `<style>
    .error-root {
      opacity: 0;
      transition: opacity linear 100ms;
      background-color: #F44336;
      color: white;
      font-size: 12px;
      font-weight: bold;
      padding: 4px;
      margin-bottom: 4px;
    }

    .show{
      visibility: visible;
      opacity: 1;
    }

    .hide{
      opacity: 0;
    }
    </style>
    <div class="error-root">
    </div>`;
        this.$root = this.querySelector(".error-root");
    }
}
customElements.define("error-message", ErrorMessage);
export class PickerElement extends HTMLElement {
    set elements(e) {
        this._elements = e;
        this.renderElements();
    }
    connectedCallback() {
        this.innerHTML = `<select></select>&nbsp;<button>Add</button>`;
        this.$select = this.querySelector("select");
        this.$button = this.querySelector("button");
        this.$button.addEventListener("click", e => {
            this.dispatchEvent(new CustomEvent("add-element", {
                bubbles: true,
                detail: this._elements[this.$select.selectedIndex]
            }));
        });
        this.renderElements();
    }
    renderElements() {
        if (this._elements) {
            this.$select.innerHTML = `
      ${this._elements.map(e => `<option value="${e.type}">${e.name}</option>`)}
    </select>`;
        }
    }
}
customElements.define("add-element-picker", PickerElement);
export class PieConfigure extends BaseElement {
    set mode(m) {
        this._mode = m;
        Array.from(this.$elements.childNodes).forEach(n => {
            n.mode = this._mode;
        });
    }
    set elements(e) {
        this.$picker.elements = e;
    }
    connectedCallback() {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            _super("connectedCallback").call(this);
            if (!this.token) {
                this.innerHTML = `Error: no token`;
                return;
            }
            if (!this.itemId) {
                this.innerHTML = `Error: no item-id`;
            }
            this.innerHTML = `<div>
      <error-message timeout="4000"></error-message>
      <add-element-picker></add-element-picker>
      <div id="elements"></div>
    </div>`;
            this.$picker = this.querySelector("add-element-picker");
            this.$picker.addEventListener("add-element", (e) => __awaiter(this, void 0, void 0, function* () {
                const { name, element, version } = e.detail;
                const existingBuildInfo = this.itemInfo.buildInfo.find(bi => {
                    return bi.name === name;
                });
                if (!existingBuildInfo) {
                    this.itemInfo.buildInfo.push(e.detail);
                }
                const dataModel = createDataModel(element, new Date().getTime().toString());
                //   element,
                //   id: new Date().getTime().toString()
                // };
                this.itemInfo.models.push(dataModel);
                yield this.client.save(this.itemInfo.id, this.itemInfo.models, this.itemInfo.buildInfo);
                this.addConfigAndPreview(dataModel);
            }));
            this.$elements = this.querySelector("#elements");
            this.$errorMessage = this.querySelector("error-message");
            const host = this.host || "http://localhost:4000";
            this.client = new ConfigureClient(host, this.token, {
                save: {
                    method: "put",
                    path: "/configure/save"
                }
            });
            try {
                this.itemInfo = yield this.client.load(this.itemId, true);
            }
            catch (e) {
                this.showError(e.message);
            }
            if (!this.itemInfo) {
                return;
            }
            log("itemInfo: ", this.itemInfo);
            const { models } = this.itemInfo;
            const elementsToWaitFor = models.map(m => `${m.element}-configure`);
            elementsToWaitFor.push(...models.map(m => m.element));
            yield this.loadJs(this.itemInfo.js.configure, elementsToWaitFor);
            const controllerLoaded = yield callPromise(this.controllerLoaded, 10, 200);
            log("controller loaded");
            log("js loaded. ready to render...");
            this.initUi();
        });
    }
    controllerLoaded() {
        log("[controllerLoaded] ? ");
        return window[CONTROLLER_GLOBAL]
            ? Promise.resolve(true)
            : Promise.reject(new Error("not ready"));
    }
    onModelUpdate(e) {
        log("model updated: ", e.detail);
        const index = this.itemInfo.models.findIndex(m => m.id === e.detail.update.id);
        if (index !== -1) {
            this.itemInfo.models.splice(index, 1, e.detail.update);
            this.client
                .save(this.itemId, this.itemInfo.models, this.itemInfo.buildInfo)
                .catch(e => {
                this.showError(e.message);
            });
        }
    }
    showError(message) {
        this.$errorMessage.message = message;
    }
    /** create the element and to layout */
    addConfigAndPreview(info) {
        const existingElement = this.$elements.querySelector(`[pie-id="${info.id}"]`);
        if (existingElement) {
            throw new Error(`An element with that id already exists: ${existingElement}`);
        }
        const cp = this.modelToConfigAndPreview(info);
        this.$elements.appendChild(cp);
    }
    removeConfigAndPreview(id) {
        const model = this.itemInfo.models.find(m => m.id === id);
        const modelIndex = this.itemInfo.models.indexOf(model);
        this.itemInfo.models.splice(modelIndex, 1);
        const elementTypeCount = this.itemInfo.models.reduce((sum, m) => {
            return sum + (m.element === model.element ? 1 : 0);
        }, 0);
        if (elementTypeCount === 0) {
            const typeIndex = this.itemInfo.buildInfo.findIndex(bi => bi.element === model.element);
            this.itemInfo.buildInfo.splice(typeIndex, 1);
        }
        const el = this.$elements.querySelector(`[pie-id="${id}"]`);
        if (el) {
            this.$elements.removeChild(el);
        }
        this.client
            .save(this.itemInfo.id, this.itemInfo.models, this.itemInfo.buildInfo)
            .then(() => {
            log("[removeConfigAndPreview] - info saved");
        })
            .catch(e => {
            log("[removeConfigAndPreview] - error saving!", e);
        });
    }
    modelToConfigAndPreview(m) {
        const el = document.createElement("config-and-preview");
        el.elementName = m.element;
        const map = window["pie-controller-pie-item"];
        if (!map) {
            throw new Error("cant find controller map as defined in pie-controllers.js");
        }
        if (!map[m.element]) {
            throw new Error(`missing controller for : ${m.element}`);
        }
        // This is so we can remove it after.
        el.setAttribute("pie-id", m.id);
        el.controller = map[m.element];
        el.model = m;
        el.mode = this._mode || "gather";
        // TODO: have autoSave flag for this default: true?
        el.addEventListener(ModelUpdatedEvent.TYPE, this.onModelUpdate.bind(this));
        el.addEventListener("delete-element", (e) => this.removeConfigAndPreview(e.detail.id));
        log("new element: ", el);
        return el;
    }
    initUi() {
        this.itemInfo.models
            .map(this.modelToConfigAndPreview.bind(this))
            .forEach(e => {
            this.$elements.appendChild(e);
        });
        this.$picker.elements = this.itemInfo.editor.elements;
    }
}
//# sourceMappingURL=configure.js.map