var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { callPromise, gapError, repeatError } from "../promise";
const resolveAfter = resolveCount => {
    let count = 0;
    return () => {
        return new Promise((resolve, reject) => {
            count += 1;
            if (count >= resolveCount) {
                resolve(true);
            }
            else {
                reject(new Error("no"));
            }
        });
    };
};
describe("call promise", () => {
    it("calls until the promise resolves", () => __awaiter(this, void 0, void 0, function* () {
        const fn = resolveAfter(10);
        return expect(callPromise(fn, 10, 10)).resolves.toEqual(true);
    }));
    it("calls promise before the limit", () => __awaiter(this, void 0, void 0, function* () {
        const fn = resolveAfter(3);
        return expect(callPromise(fn, 100, 10)).resolves.toEqual(true);
    }));
    it("returns the rejection after the call count has been executed", () => {
        const fn = resolveAfter(10);
        return expect(callPromise(fn, 2, 10)).rejects.toThrowError("no");
    });
    it("returns rejection if repeat count is less than 0", () => {
        const fn = resolveAfter(10);
        return expect(callPromise(fn, 0, 10)).rejects.toThrowError(repeatError(0).message);
    });
    it("returns rejection if repeat count is more than 1000", () => {
        const fn = resolveAfter(10);
        return expect(callPromise(fn, 1001, 10)).rejects.toThrowError(repeatError(1001).message);
    });
    it("returns rejection if gap count is <= 0", () => {
        const fn = resolveAfter(10);
        return expect(callPromise(fn, 10, 0)).rejects.toThrowError(gapError(0).message);
    });
});
//# sourceMappingURL=promise.test.js.map