var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { PlayerApiClient } from "./clients/player";
import * as debug from "debug";
import { SessionChangedEvent } from "@pie-framework/pie-player-events";
import { BaseElement } from "./base-element";
import * as logrocket from "./logrocket";
const uniq = require("lodash/uniq");
const log = debug("kds:pie-api-player");
const findOrAddSession = (data, id) => {
    const s = data.find(d => d.id === id);
    if (s) {
        return s;
    }
    const ss = { id };
    data.push(ss);
    return ss;
};
export class PiePlayer extends BaseElement {
    constructor() {
        super(...arguments);
        this.onSessionChanged = () => __awaiter(this, void 0, void 0, function* () {
            log("[onSessionChanged]");
            yield this.client.saveSession(this.playerInfo.session);
            // cant do this cos we lose the reference
            // this.playerInfo.session = sessionUpdate;
            log("saved session!");
        });
    }
    connectedCallback() {
        super.connectedCallback();
        if (!this.token) {
            this.innerHTML = `Error: no token`;
            return;
        }
        if (!this.itemId && !this.sessionId) {
            this.innerHTML = `Error: You must define either item-id or session-id.`;
            return;
        }
        // TODO: how best to configure the endpoints? For now just hardcoding.
        this.client = new PlayerApiClient(this.host, this.token, {
            saveSession: {
                method: "post",
                path: "/player/save-session"
            },
            model: {
                method: "post",
                path: "/player/model"
            },
            score: {
                method: "post",
                path: "/player/score"
            }
        });
        if (this.enableLogRocket) {
            logrocket.enable(this.logRocketId);
        }
        this.init();
    }
    get logRocketId() {
        return this.getAttribute("uuid");
    }
    get enableLogRocket() {
        return (this.getAttribute("record") !== "false" && !!this.getAttribute("record"));
    }
    get sessionId() {
        return this.getAttribute("session-id");
    }
    score(disablePartialScoring = false) {
        return __awaiter(this, void 0, void 0, function* () {
            log("[score]...");
            return this.client.score(this.playerInfo.item.id, this.playerInfo.session, disablePartialScoring);
        });
    }
    setMode(mode) {
        return __awaiter(this, void 0, void 0, function* () {
            log("[setMode] mode: ", mode);
            try {
                this.playerInfo.item.models = yield this.client.model(this.playerInfo.item.id, this.playerInfo.session, { mode });
                this.passModelToElements(this.playerInfo, false);
                return { error: null };
            }
            catch (e) {
                return { error: e.message };
            }
        });
    }
    getOrCreateElement(element, id) {
        const el = this.querySelector(`${element}[pie-id="${id}"]`);
        if (el) {
            return el;
        }
        log("[warning] automatically creating the element - because it is not in the src markup");
        const newEl = document.createElement(element);
        newEl.setAttribute("pie-id", id);
        this.appendChild(newEl);
        return newEl;
    }
    passModelToElements(result, setSession) {
        log(".. init ui... ");
        // - init the elements
        log("item: ", result.item, "session", result.session);
        result.item.models.forEach(m => {
            const el = this.getOrCreateElement(m.element, m.id);
            el.model = m;
            if (setSession) {
                el.session = findOrAddSession(result.session.data, m.id);
            }
        });
        this.addEventListener(SessionChangedEvent.TYPE, this.onSessionChanged);
    }
    init() {
        this.client
            .load(this.itemId, this.sessionId)
            .then((playerInfo) => __awaiter(this, void 0, void 0, function* () {
            this.playerInfo = playerInfo;
            log("playerInfo: ", this.playerInfo);
            if (!this.getAttribute("session-id") && this.playerInfo.session.id) {
                this.dispatchEvent(new CustomEvent("session-created", {
                    bubbles: true,
                    detail: {
                        id: this.playerInfo.session.id
                    }
                }));
            }
            yield this.loadJs(playerInfo.js.view, uniq(playerInfo.item.models.map(m => m.element)));
            this.innerHTML = playerInfo.item.markup;
            this.passModelToElements(playerInfo, true);
            log("done!");
        }))
            .catch(e => {
            this.innerHTML = e.message;
            console.error(e.stack);
        });
    }
}
//# sourceMappingURL=player-element.js.map